@@ -16,22 +16,33 @@ from sklearn.base import BaseEstimator
 from sklearn.base import MetaEstimatorMixin
 from sklearn.base import clone
 from sklearn.feature_selection import SelectorMixin
-from sklearn.model_selection import cross_val_score, GridSearchCV
+from sklearn.metrics import check_scoring
+from sklearn.model_selection import cross_val_score
 from sklearn.model_selection._search import BaseSearchCV
 from sklearn.utils.metaestimators import if_delegate_has_method
 from sklearn.utils.validation import check_is_fitted, check_X_y
 
-from featurebox.selection.mutibase import MutiBase
+from featurebox.selection.multibase import MultiBase
 from mgetool.tool import parallelize
 
-warnings.filterwarnings("ignore")
 
-
-class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
+class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MultiBase):
     """
     Exhaustion features combination.
 
-    The attribute ``estimator_`` is the model with the best feature rather than all feature combination.
+    Attributes
+    ----------
+    n_feature_: int
+        The number of selected features finally.
+
+    support_: array of shape [n_feature]
+        The mask of selected features finally.
+
+    estimator_: object
+        The best model with the best features finally (refited with all data.).
+
+    best_score_: float
+        Best score of best model of best features.
 
     Examples
     ----------
@@ -44,8 +55,7 @@ class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
     >>> bf = Exhaustion(svr,n_select=(2,),refit=True)
     >>> new_x = bf.fit_transform(X,y)
     >>> bf.support_
-    array([False, False, False, False, False, False, False, False, False,
-           False,  True, False,  True])
+    array([False, False, False,  True, False,  True, False, False])
 
     Examples
     ----------
@@ -60,46 +70,64 @@ class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
     >>> bf = Exhaustion(gd,n_select=(2,),refit=True)
     >>> new_x = bf.fit_transform(X,y)
     >>> bf.support_
-    ...
+    array([False, False, False,  True, False,  True, False, False])
+    >>> bf.best_score_
+    -0.7336740728050252
     """
 
-    def __init__(self, estimator: BaseEstimator, n_select: Tuple = (2, 3, 4), muti_grade: int = None,
-                 muti_index: List = None, must_index: List = None, n_jobs: int = 1,
-                 refit: bool = False, cv: int = 5):
+    def __init__(self, estimator: BaseEstimator, n_select: Tuple = (2, 3, 4), multi_grade: int = None,
+                 multi_index: List = None, must_index: List = None, n_jobs: int = 1,
+                 refit: bool = False, cv: int = 5,scoring: str = None):
         """
 
         Parameters
         ----------
         estimator:
-            sklearn model or GridSearchCV
+            sklearn model or GridSearchCV.
         n_select:tuple
-            the n_select list,default,n_select=(3, 4)
-        muti_grade:list
-            binding_group size, calculate the correction between binding
-        muti_index:list
-            the range of muti_grade:[min,max)
+            the n_select list,default,n_select=(3, 4).
+        multi_grade:list
+            binding_group size, calculate the correction between binding.
+        multi_index:list
+            the range of multi_grade:[min,max).
         must_index:list
-            the columns force to index
+            the columns force to index.
         n_jobs:int
-            n_jobs
+            n_jobs.
         refit:bool
             refit or not, if refit the model would used all data.
         cv:bool
-            if estimator is sklearn model, used cv, else pass
+            if estimator is sklearn model, used cv, else pass.
+        scoring:None,str
+            scoring method name.
         """
-        super().__init__(muti_grade=muti_grade, muti_index=muti_index, must_index=must_index)
-        if hasattr(estimator, "max_features") and refit:
-            print("For estimator with 'max_features' attribute, the 'max_features' would changed with "
-                  "each sub-data. that is, The 'refit estimator' which with fixed 'max_features' could be with different performance.\n"
-                  "Please change and define 'max_features' (with other parameters fixed) by manual testing after Exhaustion!!!!\n"
-                  "Please change and define 'max_features' (with other parameters fixed) by manual testing after Exhaustion!!!!",
-                  )
-        elif isinstance(estimator, BaseSearchCV) and hasattr(estimator.estimator, "max_features") and refit:
-            print("For estimator in SearchCV with 'max_features' attribute, the 'max_features' would changed with "
-                  "each sub-data. that is, The 'refit estimator' which with fixed 'max_features' could be with different performance.\n"
-                  "Please change and define 'max_features' (with other parameters fixed) by manual testing after Backforward!!!!\n"
-                  "Please change and define 'max_features' (with other parameters fixed) by manual testing after Backforward!!!!",
-                  )
+        super().__init__(multi_grade=multi_grade, multi_index=multi_index, must_index=must_index)
+        if any((hasattr(estimator, "max_features") and refit,
+                isinstance(estimator, BaseSearchCV) and hasattr(estimator.estimator, "max_features") and refit)):
+            print("For estimator with 'max_features' attribute, the 'max_features' would changed with each sub-data. \n"
+                  "Please change and define 'max_features' by SearchCV testing after Exhaustion.\n")
+        if isinstance(estimator, BaseSearchCV) and isinstance(estimator.n_jobs, int) and estimator.n_jobs > 1:
+            print(f"The 'n_jobs' for SearchCV should keep 1 or None and please pass the 'n_jobs' to {self.__class__}"
+                  f" for parallelization!")
+        if refit:
+            if isinstance(estimator, BaseSearchCV) and estimator.refit is True:
+                warnings.warn(
+                    "\nThe self.estimator_ :{} would used all the X, y data if refit! \n"
+                    "Please be careful with the 'score' and 'predict' if use, which are 'train' score/predict if inputs not changed.\n"
+                    "Check 'self.estomator_' to get CV result, such as 'self.estomator_.best_score_' for evaluation instead.".format(
+                        estimator.__class__.__name__), UserWarning)
+            else:
+                warnings.warn(
+                    "\nThe self.estimator_ :{} would used all the X, y data if refit! \n"
+                    "Please be careful with the 'score' and 'predict' if use, which are 'train' score/predict if inputs not changed.\n"
+                    "Use 'cross_val_score(self.estimator_,X[:, self.support_],y)' for evaluation instead.".format(
+                        estimator.__class__.__name__), UserWarning)
+                
+        if isinstance(estimator, BaseSearchCV):
+            estimator.scoring = scoring
+            estimator.cv = cv
+        self.scoring =scoring
+
         self.estimator = estimator
         self.score_ = []
         self.n_jobs = n_jobs
@@ -137,17 +165,12 @@ class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
                 data_x0 = x0[:, slices]
 
                 if hasattr(self.estimator, "max_features"):
-                    if self.estimator.max_features > data_x0.shape[1]:
-                        estimator_ = clone(self.estimator)
-                        estimator_.max_features = data_x0.shape[1]
-                    else:
-                        estimator_ = estimator
+                    estimator_ = clone(self.estimator)
+                    estimator_.max_features = data_x0.shape[1]
+
                 elif isinstance(self.estimator, BaseSearchCV) and hasattr(self.estimator.estimator, "max_features"):
-                    if self.estimator.estimator.max_features > data_x0.shape[1]:
-                        estimator_ = clone(self.estimator)
-                        estimator_.estimator.max_features = data_x0.shape[1]
-                    else:
-                        estimator_ = estimator
+                    estimator_ = clone(self.estimator)
+                    estimator_.estimator.max_features = data_x0.shape[1]
                 else:
                     estimator_ = estimator
 
@@ -167,7 +190,7 @@ class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
 
         self.score_ = []
         x, y = check_X_y(x, y, "csc")
-        assert all((self.check_must, self.check_muti)) in [True, False]
+        assert all((self.check_must, self.check_multi)) in [True, False]
 
         feature_list = list(range(x.shape[1]))
         fold_feature_list = self.feature_fold(feature_list)
@@ -181,25 +204,19 @@ class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
 
         feature_combination = slice_all
         index = np.argmax(scores)
-        select_feature = feature_combination[int(index)]
+        select_feature = np.array(feature_combination[int(index)])
         su = np.zeros(x.shape[1], dtype=np.bool)
         su[select_feature] = 1
         self.best_score_ = max(scores)
         self.score_ = scores
         self.support_ = su
-        self.estimator_ = clone(self.estimator)
+        self.estimator_= clone(self.estimator)
+
         if self.refit:
-            if not hasattr(self.estimator_, 'best_score_'):
-                warnings.warn(UserWarning(
-                    "The self.estimator_ :{} used all the X,y data.".format(self.estimator_.__class__.__name__),
-                    "please be careful with the later 'score' and 'predict'."))
-            if hasattr(self.estimator_, 'best_score_') and hasattr(self.estimator_, "refit") \
-                    and self.estimator_.refit is True:
-                warnings.warn(UserWarning(
-                    "The self.estimator_ :{} used all the X,y data.".format(self.estimator_.__class__.__name__),
-                    "please be careful with the later 'score' and 'predict'."))
-            if hasattr(self.estimator_, "max_features"):
-                self.estimator_.max_features = np.array(select_feature).shape[0]
+            if hasattr(self.estimator, "max_features"):
+                self.estimator_.max_features = select_feature.shape[0]
+            elif isinstance(self.estimator, BaseSearchCV) and hasattr(self.estimator.estimator, "max_features"):
+                self.estimator_.estimator.max_features = select_feature.shape[0]
             self.estimator_.fit(x[:, select_feature], y)
         self.n_feature_ = len(select_feature)
         self.score_ex = list(zip(feature_combination, scores))
@@ -227,7 +244,7 @@ class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
         return self.estimator_.predict(self.transform(X))
 
     @if_delegate_has_method(delegate='estimator_')
-    def score(self, X, y):
+    def score(self, X, y, scoring=None):
         """Reduce X to the selected feature and then return the score of the underlying estimator.
         Only available ``refit=True``.
 
@@ -238,10 +255,21 @@ class Exhaustion(BaseEstimator, MetaEstimatorMixin, SelectorMixin, MutiBase):
 
         y : array of shape [n_samples]
             The target values.
+
+        scoring : str, callable, default=None
+            Strategy to evaluate the performance of the cross-validated model on
+            the test set.
+
+            If `scoring` represents a single score, one can use:
+            a single string (see :ref:`scoring_parameter`)
+
+            The score defined by ``scoring`` if provided, and the
+            ``estimator_.score`` method otherwise else raise error.
         """
         check_is_fitted(self, 'estimator_')
-        return self.estimator_.score(self.transform(X), y)
-
+        scoring = scoring if scoring is not None else self.scoring
+        scorer = check_scoring(self.estimator_, scoring=scoring, allow_none=False)
+        return scorer(self.estimator_, self.transform(X), y)
 
     def _get_support_mask(self):
         check_is_fitted(self, 'support_')
